# -*- coding: utf-8 -*-
from django.db import models
from imagekit.models import ProcessedImageField
from lib.geo_code import GeoCode
from bbcode.models import BBCodeField
from django.urls import reverse
from gestion import fields
import uuid
import re

TYPES_LOGEMENTS = (
    ("ch", "Chambre"),
    ("st", "Studio"),
    ("co", "Colocation"),
    ("t1", "T1"),
    ("t1b", "T1 bis"),
    ("T2", "T2"),
    ("t3p", "T3 et +"),
    ("au", "Autre"),
)
NB_AVANT_OBSOLETE = 2
STATUTS_LOGEMENTS = (
    (0, "Publié"),
    (1, "Non publié"),
    (2, "Supprimé"),
    (3, "En attente de validation"),
)


class Logement(models.Model):
    date_maj = fields.DateField(verbose_name="Date de mise à jour", auto_now=True, blank=False)
    typel = models.CharField(max_length=3, verbose_name="Type de logement", choices=TYPES_LOGEMENTS)
    surface = models.PositiveIntegerField(verbose_name="Surface (m²)", blank=True, null=True)
    loyer = models.PositiveIntegerField(verbose_name="Loyer (€)", help_text="hors Charge")
    montant_charges = models.PositiveIntegerField(
        verbose_name="Montant des charges (€)",
        blank=True,
        null=True,
        help_text="Laisser vide si le montant des charges est inconnu.",
    )

    montant_caution = models.PositiveIntegerField(
        verbose_name="Montant de la caution (€)",
        blank=True,
        null=True,
        help_text="Laisser vide si le montant de la caution est inconnu.",
    )

    place_parking = models.BooleanField(verbose_name="Place de parking", default=False)

    free_date = fields.DateField(verbose_name="Date de libération")

    meuble = models.BooleanField(
        null=True,
        verbose_name="Meublé",
        help_text="un logement est qualifié de meublé s'il dispose du nécessaire en meuble et en équipements",
    )

    adresse = models.CharField(max_length=50)

    latitude = models.FloatField(editable=False, default=0)
    longitude = models.FloatField(editable=False, default=0)

    description = BBCodeField(pack="web_page")

    contact_nom = models.CharField(max_length=40, verbose_name="Nom", blank=True)
    contact_prenom = models.CharField(max_length=40, verbose_name="Prénom", blank=True)
    contact_mail = models.EmailField("Email", blank=True)
    contact_tel = models.CharField(max_length=15, verbose_name="Téléphone fixe", blank=True)
    contact_port = models.CharField(max_length=15, verbose_name="Téléphone portable", blank=True)

    pub_date = fields.DateField(verbose_name="Date de création", auto_now_add=True, editable=False)

    statut = models.IntegerField(default=0, verbose_name="Statut", choices=STATUTS_LOGEMENTS)

    nb_obsolete = models.PositiveIntegerField(editable=False, default=0)

    uuid_proprietaire = models.UUIDField(unique=True, editable=False, default=uuid.uuid4)

    def adresse_sans_ville(self):
        return re.sub(r"(31[0-9]{1,3})|toulouse", "", self.adresse, flags=re.I)

    def loyer_complet(self):
        if self.montant_charges is None:
            return self.loyer

        return self.loyer + self.montant_charges

    def maj_coords(self):
        geo = GeoCode()
        geo.geocode(self.adresse)

        if geo.latitude < 43.555 or geo.latitude > 43.647 or geo.longitude < 1.346 or geo.longitude > 1.513:
            geo.geocode(self.adresse, {"locality": "Toulouse"})

        self.latitude = geo.latitude
        self.longitude = geo.longitude

    def est_obsolete(self):
        return self.nb_obsolete >= NB_AVANT_OBSOLETE

    def __str__(self):
        return self.get_typel_display() + "—" + self.adresse + "—" + str(self.loyer) + " €"

    def get_absolute_url(self, owner=False):
        if owner:
            return reverse("logement:detail_by_uuid", args=[self.uuid_proprietaire])
        else:
            return reverse("logement:detail", args=[str(self.id)])


class LogementPhoto(models.Model):
    photo = ProcessedImageField(upload_to="photos_logements", format="JPEG", options={"quality": 75})
    logement = models.ForeignKey(Logement, related_name="photos", on_delete=models.CASCADE)
